
#    MIDAS Access Server  for  PorgamRay DCARD 

global env
source [file join $env(MIDASBASE) TclHttpd tcl MIDASDomainHandler.tcl]       ;#  load default support procedures

source [file join $env(MIDASBASE) MIDASpackages DCARDAccess1.0 pkgIndex.tcl]
package require DCARDAccess

global MIDAS_DCARDAccessServer_Verbose
set MIDAS_DCARDAccessServer_Verbose 0

# Use namespaces to isolate our methods
namespace eval urn:DCARDAccessServer {

    proc AccessServerVerbose {option} {
        do_AccessServerVerbose $option
        return [rpcvar::rpcvar int 0]
    }


    proc Set {Address Value} {
        do_Set $Address $Value
        return [rpcvar::rpcvar int 0]
    }

    proc SetBitRegister {Address Value} {
        do_SetBitRegister $Address $Value
        return [rpcvar::rpcvar int 0]
    }

    proc Get {Address} {
        set vl [do_Get $Address]
        return [rpcvar::rpcvar int $vl]
    }

    proc do_AccessServerVerbose {option} {
        global MIDAS_DCARDAccessServer_Verbose
        if {$MIDAS_DCARDAccessServer_Verbose} {puts "called method AccessServerVerbose with parameter $option"}
        set MIDAS_DCARDAccessServer_Verbose $option
        DCARDAccess_Verbose $option
    }

    proc do_Set {Address Value} {
       global MIDAS_DCARDAccessServer_Verbose

       if {$MIDAS_DCARDAccessServer_Verbose} {puts "do_Set $Address = $Value"}

       set z [catch {DCARDAccess_Set -a $Address -d $Value} m]
       if {$z == 0} then {
          return 0
       } else {
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n  $m"
       }
    }

    proc do_SetBitRegister {Address Value} {
       global MIDAS_DCARDAccessServer_Verbose

       set bits [split $Value]

       set l 0

       foreach bit $bits {
           if {[string length $bit] == 0} {continue}
           do_Set $Address $bit           ;#  write channel data bit
           do_Set [expr $Address + 2] 1   ;#  set clock
           do_Set [expr $Address + 2] 0   ;#  clear clock
           incr l
       }

       if {$MIDAS_DCARDAccessServer_Verbose} {puts "do_SetBitRegister $Address depth $l"}

       return 0
    }

    proc do_Get {Address} {
       global MIDAS_DCARDAccessServer_Verbose

       set z [catch {DCARDAccess_Get -a $Address} m]
       if {$z == 0} then {
          if {$MIDAS_DCARDAccessServer_Verbose} {puts "do_Get $Address = $m"}
          return $m
       } else {
           if {$MIDAS_DCARDAccessServer_Verbose} {puts "do_Get $Address"}
           return -code error -errorcode 0x10005 -errorinfo "received error response: rc=$z\n  $m"
       }
    }


    # We have to publish the public methods...

     SOAP::export AccessServerVerbose
     SOAP::export Set
     SOAP::export Get
}

# register this service with tclhttpd
puts [SOAP::Domain::register  -prefix  /DCARDAccessServer -namespace urn:DCARDAccessServer -uri urn:DCARDAccessServer]


proc ::SOAP::Domain::MIDAS_SOAPDomainHandler_/DCARDAccessServer {optsname sock args} {   \
    ::SOAP::Domain::MIDAS_SOAPDomainHandler_/DefaultAccessServer $optsname $sock $args}
